#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
   @project: HsPyLib-Datasource
   @package: datasource.postgres
      @file: postgres_repository.py
   @created: Sat, 12 Nov 2022
    @author: <B>H</B>ugo <B>S</B>aporetti <B>J</B>unior"
      @site: https://github.com/yorevs/hspylib
   @license: MIT - Please refer to <https://opensource.org/licenses/MIT>

   Copyright·(c)·2024,·HSPyLib
"""
from abc import abstractmethod
from datasource.crud_entity import CrudEntity
from datasource.db_repository import Connection, Cursor
from datasource.exception.exceptions import DatabaseConnectionError, DatabaseError
from datasource.redis.redis_configuration import RedisConfiguration
from hspylib.core.enums.charset import Charset
from hspylib.core.metaclass.singleton import AbstractSingleton
from hspylib.core.preconditions import check_not_none
from redis.client import Pipeline
from retry import retry
from typing import Generic, List, Optional, Tuple, TypeVar

import contextlib
import logging as log
import redis

E = TypeVar("E", bound=CrudEntity)


class RedisRepository(Generic[E], metaclass=AbstractSingleton):
    """Implementation of a data access layer for a postgres persistence store.
    Ref.: https://github.com/redis/redis-py
    Ref.: https://docs.redis.com/latest/rs/references/client_references/client_python/
    """

    def __init__(self, config: RedisConfiguration):
        self._config = config

    def __str__(self):
        return f"{self.hostname}:{self.port}/{self.database}"

    def __repr__(self):
        return str(self)

    @property
    def logname(self) -> str:
        return self.__class__.__name__.split("_", maxsplit=1)[0]

    @property
    def hostname(self) -> str:
        return self._config.hostname

    @property
    def port(self) -> int:
        return self._config.port

    @property
    def database(self) -> str:
        return self._config.database

    @property
    def password(self) -> str:
        return self._config.password

    @property
    def ssl(self) -> bool:
        return self._config.ssl

    @retry(tries=3, delay=2, backoff=3, max_delay=30)
    def _create_session(self) -> Tuple[Connection, Cursor]:
        """Connect and create a database session."""
        log.debug("%s Attempt to connect to database: %s", self.logname, str(self))
        conn = redis.Redis(ssl=self.ssl, host=self.hostname, port=self.port, password=self.password)
        log.debug("%s Connection info: %s", self.logname, conn.config_get("databases"))
        return conn, conn.pipeline()

    @contextlib.contextmanager
    def pipeline(self) -> Pipeline:
        """Generator to create a database pipeline and return it."""
        pipe = None
        try:
            _, pipe = self._create_session()
            log.debug("%s Successfully connected to database: %s [ssid=%d]", self.logname, str(self), hash(pipe))
            yield pipe
        except redis.exceptions.ConnectionError as err:
            raise DatabaseConnectionError(f"Unable to open/execute-on database session => {err}") from err
        except redis.exceptions.ResponseError as err:
            log.error("%s Pipeline failed with: %s", self.logname, err)
        except Exception as err:
            raise DatabaseError(f"{self.logname} Unable to execute pipeline -> {err}") from err
        finally:
            if pipe:
                pipe.close()

    def mget(self, *keys: str) -> List[E]:
        """Returns the values of all specified keys. For every key that does not hold a string value or does not exist,
        the special value nil is returned. Because of this, the operation never fails.
        :param keys: the list of keys to get.
        """
        check_not_none(keys)
        with self.pipeline() as pipe:
            result = []
            pipe.mget(keys)
            count = len(pipe)
            ret_val = list(filter(None, pipe.execute()))
            if ret_val:
                list(map(lambda e: result.append(self.to_entity_type(e)), ret_val[0]))
            log.debug(
                "%s Executed '%s' pipelined 'GET' command(s) and returned %d entries", self.logname, count, len(result)
            )
            return result

    def get(self, key: str) -> Optional[E]:
        """Get the value of key. If the key does not exist the special value nil is returned. An error is returned if
        the value stored at key is not a string, because GET only handles string values.
        :param key: the key to get.
        """
        check_not_none(key)
        with self.pipeline() as pipe:
            pipe.get(key)
            ret_val = list(filter(None, pipe.execute()))
            log.debug("%s Executed a pipelined 'GET' command and returned: %s", self.logname, ret_val)
            return self.to_entity_type(ret_val[0]) if ret_val else None

    def set(self, *entities: E) -> Optional[List[E]]:
        """Set key to hold the string value. If key already holds a value, it is overwritten, regardless of its type.
        Any previous time to live associated with the key is discarded on successful SET operation.
        :param entities the entities to set.
        """
        check_not_none(entities)
        with self.pipeline() as pipe:
            list(map(lambda e: pipe.set(self.build_key(e), str(e._asdict())), entities))
            count = len(pipe)
            log.debug("%s Executing '%d' pipelined 'SET' command(s)", self.logname, count)
            return pipe.execute()

    def delete(self, *keys: str) -> int:
        """Removes the specified keys. A key is ignored if it does not exist.
        :param keys: the keys to delete.
        """
        check_not_none(keys)
        with self.pipeline() as pipe:
            pipe.delete(*keys)
            ret_val = pipe.execute() or []
            log.debug("%s Executed a pipelined 'DEL' command and returned: %s", self.logname, ret_val)
            return ret_val[0] or 0

    def flushdb(self) -> None:
        """Delete all the keys of the currently selected DB. This command never fails."""
        with self.pipeline() as pipe:
            pipe.flushall()
            pipe.execute()
            log.debug("%s Executed a FLUSHDB command", self.logname)

    def keys(self, pattern: str) -> List[str]:
        """Returns all keys matching pattern.
        :param pattern: the glob-style pattern to match against.
        """
        with self.pipeline() as pipe:
            pipe.keys(pattern)
            ret_val = list(filter(None, pipe.execute()))
            result = list(map(lambda k: str(k, Charset.UTF_8.val), ret_val[0])) if ret_val else []
            log.debug("%s Executed a KEYS command and returned: %d entries", self.logname, len(ret_val))
            return result

    @abstractmethod
    def build_key(self, entity: E) -> str:
        """Builds an appropriate key for the entity.
        :param entity: the entity to build a key for.
        """

    @abstractmethod
    def to_entity_type(self, entity_string: str | bytes) -> E:
        """Convert a string into the CRUD entity.
        :param entity_string: the entity string or bytes to be converted.
        """



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
   @project: HsPyLib-Datasource
   @package: datasource
      @file: crud_entity.py
   @created: Tue, 4 May 2021
    @author: <B>H</B>ugo <B>S</B>aporetti <B>J</B>unior"
      @site: https://github.com/yorevs/hspylib
   @license: MIT - Please refer to <https://opensource.org/licenses/MIT>

   Copyright·(c)·2024,·HSPyLib
"""

from datasource.identity import Identity
from hspylib.core.namespace import Namespace
from hspylib.core.preconditions import check_argument
from typing import Any, Dict, Iterable
from uuid import UUID

import json


class CrudEntity(Namespace):
    """Generic CRUD entity type."""

    def __init__(self, entity_id: Identity | None = None, **kwargs):
        super().__init__(self.__class__.__name__, **kwargs)
        check_argument(entity_id is None or isinstance(entity_id, Identity))
        self._identity = entity_id if entity_id else Identity.auto()
        self.__iadd__(self._identity._asdict())

    @property
    def identity(self) -> Identity:
        return self._identity

    def as_dict(self) -> Dict[str, Any]:
        """Convert the entity into a dictionary object."""

        ret_dict = {}
        for key, value in zip(self.attributes, self.values):
            if isinstance(value, bool):
                ret_dict[key] = bool(value)
            elif isinstance(value, int):
                ret_dict[key] = int(value)
            elif isinstance(value, float):
                ret_dict[key] = float(value)
            elif isinstance(value, (str, UUID)):
                ret_dict[key] = str(value)
            else:
                ret_dict[key] = value.__dict__ if value else {}
        return ret_dict

    def as_json(self, indent: int = None) -> str:
        """Convert the entity into a json object."""
        return json.dumps(self.as_dict(), indent=indent)

    def as_column_set(
        self, separator: str = ", ", prefix: str | None = None, exclude: Iterable[str] | None = None
    ) -> str:
        """Return all attributes listed as tokenized 'columns = value'.
        :param separator: the separator character.
        :param prefix: an optional column prefix.
        :param exclude: a list of column names to be excluded.
        """

        column_set = []
        exclude = exclude or []
        list(
            map(
                lambda key: column_set.append(f"{key} = {(prefix or '') + key}"),
                filter(lambda a: a not in exclude, self.attributes),
            )
        )

        return separator.join(column_set)

    def as_columns(self, separator: str = ", ") -> str:
        """Return all attributes listed as tokenized 'columns'.
        :param separator: the separator character.
        """

        columns = []
        for key in self.attributes:
            columns.append(f"{key}".replace("'", ""))

        return separator.join(columns)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
   @project: HsPyLib-Datasource
   @package: datasource
      @file: crud_service.py
   @created: Tue, 4 May 2021
    @author: <B>H</B>ugo <B>S</B>aporetti <B>J</B>unior"
      @site: https://github.com/yorevs/hspylib
   @license: MIT - Please refer to <https://opensource.org/licenses/MIT>

   Copyright·(c)·2024,·HSPyLib
"""
from datasource.crud_entity import CrudEntity
from datasource.crud_repository import CrudRepository
from datasource.identity import Identity
from hspylib.core.metaclass.singleton import AbstractSingleton
from hspylib.core.namespace import Namespace
from typing import Generic, List, Optional, Set, TypeVar

E = TypeVar("E", bound=CrudEntity)
R = TypeVar("R", bound=CrudRepository)


class CrudService(Generic[R, E], metaclass=AbstractSingleton):
    """Provide a service for CRUD operations."""

    def __init__(self, repository: R):
        self._repository = repository

    @property
    def repository(self) -> R:
        return self._repository

    def list(self, filters: Namespace | None = None, order_bys: Set[str] | None = None) -> List[E]:
        """Return a list of all entries matching filters.
        :param filters: entry filters.
        :param order_bys: result set order bys.
        """
        return self.repository.find_all(filters=filters, order_bys=order_bys) or []

    def remove(self, entity: E) -> None:
        """Removes the specified entity.
        :param entity: the entity to delete.
        """
        self.repository.delete(entity)

    def get(self, identity: Identity) -> Optional[E]:
        """Gets the entity specified by ID."""
        return self.repository.find_by_id(identity)

    def save(self, entity: E) -> None:
        """Saves the specified entity.
        :param entity: the entity to save.
        """
        self.repository.save(entity)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
   @project: HsPyLib-Datasource
   @package: datasource.sqlite
      @file: sqlite_repository.py
   @created: Tue, 4 May 2021
    @author: <B>H</B>ugo <B>S</B>aporetti <B>J</B>unior"
      @site: https://github.com/yorevs/hspylib
   @license: MIT - Please refer to <https://opensource.org/licenses/MIT>

   Copyright·(c)·2024,·HSPyLib
"""
from abc import abstractmethod
from datasource.crud_entity import CrudEntity
from datasource.db_configuration import DBConfiguration
from datasource.db_repository import Connection, Cursor, DBRepository, ResultSet, Session
from datasource.exception.exceptions import DatabaseConnectionError, DatabaseError
from datasource.identity import Identity
from hspylib.core.metaclass.singleton import AbstractSingleton
from hspylib.core.namespace import Namespace
from hspylib.core.tools.text_tools import quote
from sqlite3 import Error
from typing import Generic, List, Optional, Set, Tuple, TypeVar

import contextlib
import logging as log
import sqlite3

E = TypeVar("E", bound=CrudEntity)
C = TypeVar("C", bound=DBConfiguration)


class SQLiteRepository(Generic[E, C], DBRepository[E, C], metaclass=AbstractSingleton):
    """Implementation of a data access layer for a SQLite persistence store."""

    @property
    def database(self) -> str:
        return self._config.database

    @property
    def info(self) -> str:
        return f"{self.database}"

    def _create_session(self) -> Tuple[Connection, Cursor]:
        log.debug("%s Attempt to connect to database: %s", self.logname, str(self))
        conn = sqlite3.connect(self.database)
        return conn, conn.cursor()

    @contextlib.contextmanager
    def _session(self) -> Session:
        log.debug("%s Attempt to connect to database: %s", self.logname, self.database)
        conn, dbs = None, None
        try:
            conn, dbs = self._create_session()
            log.debug("%s Successfully connected to database: %s [ssid=%d]", self.logname, self.info, hash(dbs))
            yield dbs
        except Error as err:
            raise DatabaseConnectionError(f"Unable to open/execute-on database: {self.database} => {err}") from err
        finally:
            if conn:
                log.debug("%s Closing connection [ssid=%d]", self.logname, hash(dbs))
                conn.commit()
                conn.close()

    def execute(self, sql_statement: str, **kwargs) -> Tuple[int, Optional[ResultSet]]:
        with self._session() as dbs:
            try:
                args = dict(kwargs)
                log.debug(
                    "%s Executing SQL statement [ssid=%s]:\n" "\t|-Arguments: %s\n" "\t|-Statement: %s",
                    self.logname,
                    hash(dbs),
                    args,
                    sql_statement,
                )
                rows = dbs.execute(sql_statement, tuple(args.values())).fetchall()
                return dbs.rowcount, rows
            except (sqlite3.ProgrammingError, sqlite3.OperationalError) as err:
                raise DatabaseError(f"Unable to execute statement => {sql_statement}") from err

    def count(self) -> int:
        sql = f"SELECT COUNT(*) FROM {self.table_name()}"
        return int(self.execute(sql)[1][0][0])

    def delete(self, entity: E) -> None:
        self.delete_by_id(entity.identity)

    def delete_by_id(self, entity_id: Identity) -> None:
        clauses = [f"{k} = {quote(v)}" for k, v in zip(entity_id.attributes, entity_id.values)]
        sql = f"DELETE FROM {self.table_name()} WHERE " + " AND ".join(clauses)
        self.execute(sql)

    def delete_all(self, entities: List[E]) -> None:
        values, s = [], entities[0]
        list(map(lambda e: values.append(str(e.values)), entities))
        sql = f"DELETE FROM " f"{self.table_name()} WHERE ({s.as_columns()}) IN ({', '.join(values)}) "
        self.execute(sql)

    def save(self, entity: E) -> None:
        columns, ids = entity.as_columns(), set(entity.identity.attributes)
        sql = (
            f"INSERT INTO "
            f"{self.table_name()} ({columns}) VALUES {entity.values} "
            f"ON CONFLICT DO UPDATE SET {entity.as_column_set(prefix='EXCLUDED.', exclude=ids)}"
        )
        self.execute(sql)

    def save_all(self, entities: List[E]) -> None:
        values, sample = [], entities[0]
        columns, ids = sample.as_columns(), set(sample.identity.attributes)
        list(map(lambda e: values.append(str(e.values)), entities))
        sql = (
            f"INSERT INTO "
            f"{self.table_name()} ({columns}) VALUES {', '.join(values)} "
            f"ON CONFLICT DO UPDATE SET {sample.as_column_set(prefix='EXCLUDED.', exclude=ids)}"
        )
        self.execute(sql)

    def find_all(
        self,
        columns: Optional[Set[str]] = None,
        filters: Optional[Namespace] = None,
        order_bys: Optional[List[str]] = None,
        limit: int = 500,
        offset: int = 0,
    ) -> List[E]:
        columns = "*" if not columns else ", ".join(columns)
        clauses = list(filter(None, filters.values)) if filters else None
        orders = list(filter(None, order_bys)) if order_bys else None
        sql = (
            f"SELECT {columns} FROM {self.table_name()} "
            f"{('WHERE ' + ' AND '.join(clauses)) if clauses else ''} "
            f"{('ORDER BY ' + ', '.join(orders)) if orders else ''} "
            f"LIMIT {limit} OFFSET {offset}"
        )

        return list(map(self.to_entity_type, self.execute(sql)[1]))

    def find_by_id(self, entity_id: Identity, columns: Optional[Set[str]] = None) -> Optional[E]:
        columns = "*" if not columns else ", ".join(columns)
        clauses = [f"{k} = {quote(v)}" for k, v in zip(entity_id.attributes, entity_id.values)]
        sql = f"SELECT {columns} FROM {self.table_name()} " f"WHERE {' AND '.join(clauses)}"
        result = next((e for e in self.execute(sql)[1]), None)

        return self.to_entity_type(result) if result else None

    def exists_by_id(self, entity_id: Identity) -> bool:
        clauses = [f"{k} = {quote(v)}" for k, v in zip(entity_id.attributes, entity_id.values)]
        sql = f"SELECT EXISTS(SELECT 1 FROM {self.table_name()} WHERE {' AND '.join(clauses)})"

        return self.execute(sql)[1][0][0] > 0

    @abstractmethod
    def table_name(self) -> str:
        pass

    @abstractmethod
    def to_entity_type(self, entity_dict: dict | tuple) -> E:
        pass



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
   @project: HsPyLib-Datasource
   @package: datasource.firebase
      @file: firebase_repository.py
   @created: Tue, 4 May 2021
    @author: <B>H</B>ugo <B>S</B>aporetti <B>J</B>unior"
      @site: https://github.com/yorevs/hspylib
   @license: MIT - Please refer to <https://opensource.org/licenses/MIT>

   Copyright·(c)·2024,·HSPyLib
"""

from abc import abstractmethod
from datasource.crud_entity import CrudEntity
from datasource.firebase.firebase_configuration import FirebaseConfiguration
from datasource.identity import Identity
from hspylib.core.metaclass.singleton import AbstractSingleton
from hspylib.core.namespace import Namespace
from hspylib.core.preconditions import check_not_none
from hspylib.modules.fetch.fetch import delete, get, put
from hspylib.modules.fetch.http_response import HttpResponse
from requests.exceptions import HTTPError
from typing import Any, Generic, List, Optional, TypeVar

import json
import logging as log

E = TypeVar("E", bound=CrudEntity)


class FirebaseRepository(Generic[E], metaclass=AbstractSingleton):
    """Implementation of a data access layer for a Firebase persistence store.
    API   Ref.: https://firebase.google.com/docs/reference/rest/database
    Auth  Ref.: https://firebase.google.com/docs/database/rest/auth#python
    Order Ref:. https://firebase.google.com/docs/database/rest/retrieve-data#section-rest-ordered-data
    """

    @staticmethod
    def _assert_response(response: HttpResponse, message: str) -> HttpResponse:
        """Assert that the specified response code is from 2xx.
        :param response: the http response object.
        :param message: the error message to be raised if the response code is from 2xx.
        """
        check_not_none(response, "Response is none")
        if not response.status_code.is_2xx():
            raise HTTPError(f"{response.status_code} => {message}")
        return response

    @staticmethod
    def quote(value: Any) -> str:
        """Quote or double quote the value according to the value type.
        :param value: the value to be quoted.
        """
        if isinstance(value, bool):
            return f"{str(value).lower()}"
        if isinstance(value, int | float):
            return str(value)

        return f'"{value}"' if value.startswith("'") and value.endswith("'") else f"'{value}'"

    def __init__(self, config: FirebaseConfiguration):
        self._payload = None
        self._config = config

    def __str__(self) -> str:
        return str(self._config) + f"TABLE_NAME={self.table_name() or ''}"

    def __repr__(self) -> str:
        return str(self)

    @property
    def logname(self) -> str:
        return self.__class__.__name__.split("_", maxsplit=1)[0]

    @property
    def config(self) -> FirebaseConfiguration:
        return self._config

    @property
    def info(self) -> str:
        return f"{self.config}"

    @property
    def hostname(self) -> str:
        return self._config.hostname

    @property
    def port(self) -> int:
        return self._config.port

    @property
    def base_url(self) -> str:
        return self._config.base_url

    @property
    def database(self) -> str:
        return self._config.database

    def delete(self, entity: E) -> None:
        """Deletes a given entity.
        :param entity: the entity to delete.
        """
        self.delete_by_id(entity.identity)

    def delete_by_id(self, entity_id: Identity) -> None:
        """Deletes the entity with the given id.
        :param entity_id: the ID of the entity to be deleted.
        """
        ids = ".".join(entity_id.values)
        url = f"{self._config.url(self.table_name())}/{ids}.json"
        log.debug("Deleting firebase entry: \n\t|-Id=%s\n\t|-From %s", entity_id, url)
        self._assert_response(delete(url), f"Unable to delete from={url}")

    def delete_all(self, entities: List[E]) -> None:
        """Deletes all given entities.
        TODO Check if there is a better way of doing it.
        :param entities: the entities to be deleted.
        """
        list(map(self.delete, entities))

    def save(self, entity: E) -> None:
        """Saves a given entity.
        :param entity: the entity to save.
        """
        ids = ".".join(entity.identity.values)
        url = f"{self._config.url(self.table_name())}/{ids}.json"
        payload = entity.as_json()
        log.debug("Saving firebase entry: \n\t|-%s \n\t|-Into %s", entity, url)
        self._assert_response(put(url, payload), f"Unable to put into={url} with json_string={payload}")

    def save_all(self, entities: List[E]) -> None:
        """Saves all given entities.
        TODO Check if there is a better way of doing it.
        :param entities: the entities to be saved.
        """
        list(map(self.save, entities))

    def find_all(
        self,
        order_by: List[str] | None = None,
        limit_to_first: int | None = None,
        limit_to_last: int | None = None,
        start_at: int | str | None = None,
        end_at: int | str | None = None,
        equal_to: int | str | None = None,
    ) -> List[E]:
        """Returns all entities of the type.
        :param order_by: result set order by.
        :param limit_to_first: the maximum number of entries to be fetch (first N entries).
        :param limit_to_last: the maximum number of entries to be fetch (last N entries).
        :param start_at: arbitrary starting points for queries.
        :param end_at: arbitrary ending points for your queries.
        :param equal_to: the maximum number of entries to be fetch.
        """

        f_order_by = "orderBy=" + (",".join([f'"{o}"' for o in order_by]) if order_by else '"$key"')
        f_start_at = f"&startAt={self.quote(start_at)}" if start_at else ""
        f_end_at = f"&endAt={self.quote(end_at)}" if end_at else ""
        f_equal_to = f"&equalTo={self.quote(equal_to)}" if equal_to else ""
        f_limit_first = f"&limitToFirst={limit_to_first}" if limit_to_first else ""
        f_limit_last = f"&limitToLast={limit_to_last}" if limit_to_last else ""
        url = (
            f"{self._config.url(self.table_name())}.json?"
            f"{f_order_by}{f_start_at}{f_end_at}{f_equal_to}{f_limit_first}{f_limit_last}"
        )
        log.debug("Fetching firebase entries: \n\t|-From %s", url)
        response = self._assert_response(get(url), f"Unable to get from={url}")
        if response.body and response.body != "null":
            return self.to_entity_list(response.body)

        return []

    def find_by_id(self, entity_id: Identity) -> Optional[E]:
        """Return the entity specified by ID from the Firebase store, None if no such entry is found."""
        ids = ".".join(entity_id.values)
        url = f"{self._config.url(self.table_name())}/{ids}.json"
        log.debug("Fetching firebase entry: \n\t|-Id=%s\n\t|-From %s", entity_id, url)
        response = self._assert_response(get(url), f"Unable to get from={url}")
        if response.body and response.body != "null":
            return self.to_entity_type(json.loads(response.body))

        return None

    def exists_by_id(self, entity_id: Identity) -> bool:
        """Returns whether an entity with the given id exists.
        TODO Check if there is a better way of doing it.
        :param entity_id: the entity ID.
        """
        return self.find_by_id(entity_id) is not None

    @abstractmethod
    def to_entity_type(self, entity_dict: dict | tuple) -> E:
        """Convert a dict or tuple, generally from a result set, into the CRUD entity.
        :param entity_dict: the entity dict or tuple to be converted.
        """

    @abstractmethod
    def table_name(self) -> str:
        """Return the table name (repository name)."""

    def to_entity_list(self, json_string: str, filters: Namespace | None = None) -> List[E]:
        """Return filtered entries from the json_string as a list.
        :param json_string: the json string to be parsed.
        :param filters: entry filters.
        """
        ret_list = []
        for value in json.loads(json_string).values():
            if not filters or all(k in value and value[k] == v for k, v in filters.items()):
                ret_list.append(self.to_entity_type(value))
        return ret_list



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
   @project: HsPyLib-Datasource
   @package: datasource.firebase
      @file: firebase_configuration.py
   @created: Tue, 4 May 2021
    @author: <B>H</B>ugo <B>S</B>aporetti <B>J</B>unior"
      @site: https://github.com/yorevs/hspylib
   @license: MIT - Please refer to <https://opensource.org/licenses/MIT>

   Copyright·(c)·2024,·HSPyLib
"""
from hspylib.core.config.app_config import AppConfigs


class RedisConfiguration(AppConfigs):
    """Represents a Redis datasource configuration."""

    def __init__(self, resource_dir: str, filename: str | None = None, profile: str | None = None):
        super().__init__(resource_dir, filename, profile)
        self._hostname = self["datasource.hostname"]
        self._database = self["datasource.database"]
        self._port = int(self["datasource.port"])
        self._username = self["datasource.username"]
        self._password = self["datasource.password"]
        self._ssl = self["datasource.ssl"] or False

    @property
    def hostname(self) -> str:
        return self._hostname

    @property
    def port(self) -> int:
        return self._port

    @property
    def database(self) -> str:
        return self._database

    @property
    def password(self) -> str:
        return self._password

    @property
    def ssl(self) -> bool:
        return self._ssl



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
   @project: HsPyLib-Datasource
   @package: datasource
      @file: identity.py
   @created: Thu, 03 Nov 2022
    @author: <B>H</B>ugo <B>S</B>aporetti <B>J</B>unior"
      @site: https://github.com/yorevs/hspylib
   @license: MIT - Please refer to <https://opensource.org/licenses/MIT>

   Copyright·(c)·2024,·HSPyLib
"""
from collections import namedtuple
from hspylib.core.namespace import Namespace
from hspylib.core.preconditions import check_argument
from typing import Any, Dict, Tuple, Type, Union
from uuid import UUID, uuid4

import random
import string
import sys

IDENTITY = Union[Tuple, int, str, UUID]


class Identity(Namespace):
    """Represent a unique identity of an entity."""

    @classmethod
    def auto(cls, field_name: str = "id", id_type: Type = UUID) -> "Identity":
        """Automatically assigns a value for the identity.
        :param field_name: the identity field name.
        :param id_type: the type of the identity.
        """
        _id_ = namedtuple("Identity", [f"{field_name}"])
        match id_type.__name__:
            case "int":
                number = random.randint(0, sys.maxsize)
                return Identity(_id_(number))
            case "str":
                letters = string.ascii_lowercase + "-_"
                st = "".join(random.choice(letters) for _ in range(32))
                return Identity(_id_(st))
            case "UUID":
                return Identity(_id_(uuid4().hex))
            case other:
                raise NotImplementedError(f"auto-identity generator for type '{other}' is not implemented")

    def __init__(self, identity: Tuple | Namespace | "Identity"):
        check_argument(isinstance(identity, Tuple | Identity | Namespace), "Must be a named tuple or Identity")
        identity = identity.ID if isinstance(identity, Identity) else identity
        self._identity = identity
        super().__init__("Identity", **self._asdict())

    def __str__(self) -> str:
        return str(self._identity)

    @property
    def ID(self) -> Tuple:
        return self._identity

    def _asdict(self) -> Dict[str, Any]:
        """Return a new dict which maps field names to their values."""
        return self._identity._asdict()

    def as_column_set(self, separator: str = ",") -> str:
        """Return all attributes listed as tokenized 'columns = value'.
        :param separator: the separator character.
        """
        column_set = []
        list(map(lambda key, value: column_set.append(f"{key} = {value}"), self.attributes, self.values))
        return separator.join(column_set)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
   @project: HsPyLib-Datasource
   @package: datasource
      @file: __classpath__.py
   @created: Fri, 23 Dec 2022
    @author: "<B>H</B>ugo <B>S</B>aporetti <B>J</B>unior")"
      @site: "https://github.com/yorevs/hspylib")
   @license: MIT - Please refer to <https://opensource.org/licenses/MIT>

   Copyright·(c)·2024,·HSPyLib
"""

from hspylib.core.metaclass.classpath import Classpath
from hspylib.core.tools.commons import get_path, run_dir


class _Classpath(Classpath):
    """Provide a class to help locating user-defined classes, packages, sources and resources."""

    def __init__(self):
        super().__init__(get_path(__file__), get_path(run_dir()), (get_path(__file__) / "resources"))


# Instantiate the classpath singleton
assert (classpath := _Classpath().INSTANCE) is not None, "Failed to create Classpath instance"
